# -*- coding: utf-8 -*-

import os
import shutil
import subprocess

from . import tools

TMP_FILE = '/tmp/confupdater.tmp'

def replace_file_line(filename, oldline, newline):
    with open(filename, 'r') as conf, open(TMP_FILE, 'w') as tmp:
        for line in conf:
            # Find line starting with oldline
            if line.startswith(oldline):
                # Replace te line with oldline, newline, \n (to not repeat the oldline in newline)
                tmp.write(oldline)
                tmp.write(newline)
                tmp.write('\n')
                # Dump the rest of the file and break the loop
                tmp.write(conf.read())
                break
            else:
                tmp.write(line)
    # Copy the file contents to the original file (preserves permissions of the original file)
    shutil.copyfile(TMP_FILE, filename)
    os.unlink(TMP_FILE)

def run_mysql_query(query, database):
    maria_started = tools.is_service_started('mariadb')
    if not maria_started:
        tools.start_service('mariadb')
    subprocess.run(['docker', 'exec', '-i', 'mariadb', 'mysql', '-e', query, database])
    if not maria_started:
        tools.stop_service('mariadb')

def app_exists(app):
    return os.path.exists(os.path.join('/srv/', app))

def update_gmaps_api_key(api_key):
    # CKAN
    if app_exists('ckan'):
        replace_file_line('/srv/ckan/conf/ckan.ini', 'ckanext.geoview.gapi_key = ', api_key)
    # Crisis Cleanup
    if app_exists('crisiscleanup'):
        replace_file_line('/srv/crisiscleanup/conf/boot.rb', 'ENV[\'GOOGLE_MAPS_API_KEY\'] = ', api_key)
    # Pan.do/ra
    if app_exists('pandora'):
        replace_file_line('/srv/pandora/conf/local_settings.py', 'GOOGLE_API_KEY = ', '\'{}\''.format(api_key))
    # Sahana
    if app_exists('sahana'):
        replace_file_line('/srv/sahana/conf/000_config.py', 'settings.gis.api_google = ', '"{}"'.format(api_key))
    # Sahana Demo
    if app_exists('sahana-demo'):
        replace_file_line('/srv/sahana-demo/conf/000_config.py', 'settings.gis.api_google = ', '"{}"'.format(api_key))
    # SAMBRO
    if app_exists('sambro'):
        replace_file_line('/srv/sambro/conf/000_config.py', 'settings.gis.api_google = ', '"{}"'.format(api_key))
    # Sigmah
    if app_exists('sigmah'):
        replace_file_line('/srv/sigmah/conf/sigmah.properties', 'maps.key=', api_key)
    # Ushahidi
    if app_exists('ushahidi'):
        replace_file_line('/srv/ushahidi/conf/config.json', '    "google_analytics_id": ', '"{}"'.format(api_key))

def update_email(email):
    # CKAN
    if app_exists('ckan'):
        replace_file_line('/srv/ckan/conf/ckan.ini', 'smtp.mail_from = ', email)
        replace_file_line('/srv/ckan-datapusher/conf/datapusher_settings.py', 'FROM_EMAIL = ', '\'{}\''.format(email))
    # Crisis Cleanup
    if app_exists('crisiscleanup'):
        replace_file_line('/srv/crisiscleanup/conf/initializers/devise.rb', '  config.mailer_sender = ', '\'{}\''.format(email))
    # CTS
    if app_exists('cts'):
        replace_file_line('/srv/cts/conf/spotter.py', 'SERVER_EMAIL = ', '\'{}\''.format(email))
    # GNU Health
    if app_exists('gnuhealth'):
        replace_file_line('/srv/gnuhealth/conf/trytond.conf', 'from = ', email)
    # KanBoard
    if app_exists('kanboard'):
        replace_file_line('/srv/kanboard/conf/config.php', 'define(\'MAIL_FROM\', ', '\'{}\');'.format(email))
    # Mifos X
    if app_exists('mifosx'):
        query = 'UPDATE `c_external_service_properties` SET `value` = "{}" WHERE `external_service_id` = 2 and `name` LIKE "username";'.format(email)
        run_mysql_query(query, 'mifostenant-default')
    # Sahana
    if app_exists('sahana'):
        replace_file_line('/srv/sahana/conf/000_config.py', 'settings.mail.sender = ', '"{}"'.format(email))
        replace_file_line('/srv/sahana/conf/000_config.py', 'settings.mail.approver = ', '"{}"'.format(email))
    # Sahana Demo
    if app_exists('sahana-demo'):
        replace_file_line('/srv/sahana-demo/conf/000_config.py', 'settings.mail.sender = ', '"{}"'.format(email))
        replace_file_line('/srv/sahana-demo/conf/000_config.py', 'settings.mail.approver = ', '"{}"'.format(email))
    # SAMBRO
    if app_exists('sambro'):
        replace_file_line('/srv/sambro/conf/000_config.py', 'settings.mail.sender = ', '"{}"'.format(email))
        replace_file_line('/srv/sambro/conf/000_config.py', 'settings.mail.approver = ', '"{}"'.format(email))
    # SeedDMS
    if app_exists('seeddms'):
        replace_file_line('/srv/seeddms/conf/settings.xml', '        <smtp smtpServer="postfix" smtpPort="25" smtpSendFrom=', '"{}" smtpUser="" smtpPassword=""/>'.format(email))
    # Sigmah
    if app_exists('sigmah'):
        replace_file_line('/srv/sigmah/conf/sigmah.properties', 'mail.from.address=', email)
        replace_file_line('/srv/sigmah/conf/sigmah.properties', 'mail.support.to=', email)
    # Ushahidi
    if app_exists('ushahidi'):
        email_json = '{\\"incoming_type\\":\\"IMAP\\",\\"incoming_server\\":\\"localhost\\",\\"incoming_port\\":143,\\"incoming_security\\":\\"None\\",\\"incoming_username\\":\\"{}\\",\\"incoming_password\\":\\"password\\",\\"outgoing_type\\":\\"SMTP\\",\\"outgoing_server\\":\\"postfix\\",\\"outgoing_port\\":25,\\"outgoing_security\\":\\"None\\",\\"outgoing_username\\":\\"{}\\",\\"outgoing_password\\":\\"password\\",\\"from\\":\\"{}\\",\\"from_name\\":\\"Ushahidi\\"}'
        query = 'UPDATE `config` SET `config_value` = "{}" WHERE `group_name` LIKE "data-provider" AND `config_key` LIKE "email";'.format(email_json)
        run_mysql_query(query, 'ushahidi')

def update_url(host):
    # CKAN
    if app_exists('ckan'):
        replace_file_line('/srv/ckan/conf/ckan.ini', 'ckan.site_url = ', 'https://ckan.{}'.format(host))
    # Motech
    if app_exists('motech'):
        replace_file_line('/srv/motech/conf/config/motech-settings.properties', 'server.url=', 'https://motech.{}'.format(host))
    # Pan.do/ra
    if app_exists('pandora'):
        replace_file_line('/srv/pandora/conf/config.jsonc', '        "url": ', '"pandora.{}"'.format(host))
    # Sahana
    if app_exists('sahana'):
        replace_file_line('/srv/sahana/conf/000_config.py', 'settings.base.public_url = ', '"https://sahana.{}"'.format(host))
    # Sahana Demo
    if app_exists('sahana-demo'):
        replace_file_line('/srv/sahana-demo/conf/000_config.py', 'settings.base.public_url = ', '"https://sahana-demo.{}"'.format(host))
    # SAMBRO
    if app_exists('sambro'):
        replace_file_line('/srv/sambro/conf/000_config.py', 'settings.base.public_url = ', '"https://sambro.{}"'.format(host))
    # Ushahidi
    if app_exists('ushahidi'):
        replace_file_line('/srv/ushahidi/conf/config.json', '    "backend_url": ', '"https://ush.{}/platform",'.format(host))
        api_url = '\\"https:\\\\/\\\\/ush.{}\\\\/platform\\\\/api\\\\/v3\\\\/config\\\\/data-provider\\"'.format(host)
        query = 'UPDATE `config` SET `config_value` = "{}" WHERE `group_name` LIKE "data-provider" AND `config_key` LIKE "url";'.format(api_url)
        run_mysql_query(query, 'ushahidi')
